import * as React from 'react'
import {TextMenu} from '../menus/TextMenu'
const ReactTestRenderer = require('react-test-renderer');

const mockFunction = jest.fn();
const node_setup = {
    family: 'Arial',
    size: 14,
    color: '#00FF00',
    background_color: '#FFFFFF',
    is_italic: false,
    is_bold: true,
    shape_type: 0,
};
const lang = {
    ['LC__CMDB__CATG__SMVIEW_MAPS__FONT_STYLE']: 'Font Style',
    ['LC__CMDB__CATG__SMVIEW_MAPS__FONT_SIZE']: 'Font Size',
    ['LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_BOLD']: 'Is Bold',
    ['LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_ITALIC']: 'Is Italic',
    ['LC__CMDB__CATG__SMVIEW_MAPS__TEXT_COLOR']: 'Text Color',
    ['LC__CMDB__CATG__SMVIEW_MAPS__BACKGROUND_COLOR']: 'Background Color'
};
const component = ReactTestRenderer.create(
    <TextMenu node_setup={node_setup} onNodeSetupChange={mockFunction} language={lang} />,
);
const instance = component.root;

it('renders text menu correctly', () => {
    expect(component).toMatchSnapshot();
});

it('changes font family', () => {
    let select = instance.findByProps({name: "font_family"});
    select.props.onChange({
        currentTarget: { value: 'Tahoma' }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 14,
        color: '#00FF00',
        background_color: '#FFFFFF',
        is_italic: false,
        is_bold: true,
        shape_type: 0,
    }, true);
});

it('changes font size', () => {
    let select = instance.findByProps({name: "font_size"});
    select.props.onChange({
        currentTarget: { value: '28' }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 28,
        color: '#00FF00',
        background_color: '#FFFFFF',
        is_italic: false,
        is_bold: true,
        shape_type: 0,
    }, true);
});

it('changes is italic', () => {
    let select = instance.findByProps({name: "is_italic"});
    select.props.onChange({
        currentTarget: { checked: true }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 28,
        color: '#00FF00',
        background_color: '#FFFFFF',
        is_italic: true,
        is_bold: true,
        shape_type: 0,
    }, true);
});

it('changes is bold', () => {
    let select = instance.findByProps({name: "is_bold"});
    select.props.onChange({
        currentTarget: { checked: false }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 28,
        color: '#00FF00',
        background_color: '#FFFFFF',
        is_italic: true,
        is_bold: false,
        shape_type: 0,
    }, true);
});

it('changes text color', () => {
    let select = instance.findByProps({name: "text_color"});
    select.props.onChange({
        currentTarget: { value: '#FFFF00' }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 28,
        color: '#FFFF00',
        background_color: '#FFFFFF',
        is_italic: true,
        is_bold: false,
        shape_type: 0,
    }, true);
});

it('changes background color', () => {
    let select = instance.findByProps({name: "background_color"});
    select.props.onChange({
        currentTarget: { value: '#00AA00' }
    });
    expect(mockFunction).toHaveBeenCalledWith({
        family: 'Tahoma',
        size: 28,
        color: '#FFFF00',
        background_color: '#00AA00',
        is_italic: true,
        is_bold: false,
        shape_type: 0,
    }, true);
});
