import {
  CreateEdgeInputMode,
  EdgeEventArgs,
  INode
} from 'yfiles'
import { EdgeSetupProps, MapConnection, MapObject } from '../MapsCompose'
import { api } from '../ajax'
import { EdgeStyleSettings } from '../styles/EdgeStyleSettings'
import { ApiProps } from './NodeEditorInputMode'

export class EdgeCreationInputMode extends CreateEdgeInputMode {
    private readonly apiProps: ApiProps;
    private editorProps: EdgeSetupProps;

    constructor (apiProps: ApiProps, editorProps: EdgeSetupProps) {
      super()
      this.enabled = true
      this.apiProps = apiProps
      this.editorProps = editorProps
    }

    updateEditorProps (props: EdgeSetupProps) {
      this.editorProps = props
    }

    onEdgeCreated (evt: EdgeEventArgs) {
      super.onEdgeCreated(evt)

      // get depending objects
      const edge = evt.item
      const sourceNode = evt.sourcePortOwner as INode
      const targetNode = evt.targetPortOwner as INode
      const sourceMapObject: MapObject = sourceNode.tag as MapObject
      const targetMapObject: MapObject = targetNode.tag as MapObject
      let bends = ''
      edge.bends.forEach((bend) => {
        if (bends.length > 0) {
          bends += ';'
        }
        bends += parseInt(bend.location.x.toString()) + ',' + parseInt(bend.location.y.toString())
      })

      // create map connection object
      const connection: MapConnection = {
        mapconnection_id: -1,
        source_shd_map_mapobject__id: sourceMapObject.mapobject_id,
        target_shd_map_mapobject__id: targetMapObject.mapobject_id,
        textvisible: '0',
        title: '',
        bends: bends,
        sourcearrowtype: this.editorProps.source_arrow_type.toString(),
        targetarrowtype: this.editorProps.target_arrow_type.toString(),
        linecolor: this.editorProps.line_color,
        linewidth: this.editorProps.stroke.toString(),
        linestyle: this.editorProps.style.toString(),
        realizertype: ''
      }

      // assign map connection object to edge
      edge.tag = connection

      // set edge style based on map connection object (to avoid using default edge style)
      this.apiProps.graph_component.graph.setStyle(edge, EdgeStyleSettings.getEdgeStyle(connection))

      // save new edge
      api('shd.map_connection.create',
        {
          shd_map__id: this.apiProps.map_id,
          ...connection
        },
        this.apiProps).then(res => {
        // save map connection ID
        connection.mapconnection_id = parseInt(res.result.response)
        edge.tag = connection
      })
    }
}
