import {
  IBend,
  IEdge,
  InputModeEventArgs,
  MoveInputMode
} from 'yfiles'
import { MapConnection } from '../MapsCompose'
import { ApiProps } from './NodeEditorInputMode'
import { EdgeStyleSettings } from '../styles/EdgeStyleSettings'
import { api } from '../ajax'

export class EdgeMovingInputMode extends MoveInputMode {
    private readonly apiProps: ApiProps;

    constructor (apiProps: ApiProps) {
      super()
      this.enabled = true
      this.apiProps = apiProps
    }

    onDragFinished (evt: InputModeEventArgs) {
      super.onDragFinished(evt)

      if (this.apiProps.graph_component.selection.selectedEdges.size <= 0) {
        return
      }

      this.apiProps.graph_component.selection.selectedEdges.forEach((edge: IEdge) => {
        let bends = ''
        edge.bends.forEach((bend: IBend) => {
          if (bends.length > 0) {
            bends += ';'
          }
          bends += parseInt(bend.location.x.toString()) + ',' + parseInt(bend.location.y.toString())
        })

        const connection = edge.tag as MapConnection
        connection.bends = bends

        // assign map connection object to edge
        edge.tag = connection

        // set edge style based on map connection object (to avoid using default edge style)
        this.apiProps.graph_component.graph.setStyle(edge, EdgeStyleSettings.getEdgeStyle(connection))

        // update edge
        api('shd.map_connection.update',
          {
            mapconnection_id: connection.mapconnection_id,
            bends: bends
          },
          this.apiProps).then(() => {
        })
      })
    }
}
