import { ILabel, INode, InputModeEventArgs, MoveLabelInputMode } from 'yfiles'
import { MapObject } from '../MapsCompose'
import { api } from '../ajax'
import { ApiProps } from './NodeEditorInputMode'
import { NodeStyleSettings } from '../styles/NodeStyleSettings'

export class LabelPositionInputMode extends MoveLabelInputMode {
    private readonly apiProps: ApiProps;

    constructor (apiProps: ApiProps) {
      super()
      this.enabled = true
      this.apiProps = apiProps
    }

    onDragFinished (evt: InputModeEventArgs) {
      super.onDragFinished(evt)

      const label: ILabel = this.apiProps.graph_component.selection.selectedLabels.first()
      const node: INode = label.owner as INode
      const labelGeometry = label.layoutParameter.model.getGeometry(label, label.layoutParameter)
      const modelPosition = NodeStyleSettings.getLabelPosition(node.layout.center, labelGeometry.orientedRectangleCenter)
      const position = NodeStyleSettings.getIndexOfPosition(modelPosition)

      const mapObject: MapObject = node.tag as MapObject
      mapObject.labelposition = position.toString()

      // assign map object object to node
      node.tag = mapObject

      // update node
      api('shd.map_object.update',
        {
          mapobject_id: mapObject.mapobject_id,
          labelposition: position
        },
        this.apiProps).then(() => {
      })
    }
}
