import * as React from 'react'
import { ShdMap, TransferObject } from '../MapsCompose'

interface MapMenuProps {
    maps: ShdMap[],
    language: {[key: string]: string},
}

interface MapMenuState {
    filter: string,
    filtered_items: ShdMap[],
}

export class MapMenu extends React.Component<MapMenuProps, MapMenuState> {
  constructor (props: MapMenuProps) {
    super(props)
    this.handleDragStart = this.handleDragStart.bind(this)
    this.handleFilterChange = this.handleFilterChange.bind(this)
    this.state = {
      filter: '',
      filtered_items: this.props.maps
    }
  }

  handleDragStart (evt: React.DragEvent<HTMLElement>) {
    const id = (evt.target as HTMLElement).id
    let map: ShdMap
    for (const key in this.props.maps) {
      if (this.props.maps.hasOwnProperty(key)) {
        const object = this.props.maps[key]
        if (object.map_id == parseInt(id)) {
          map = object
        }
      }
    }

    const transfer: TransferObject = {
      transfer_type: 'map',
      object_id: parseInt(id),
      object_title: map.title,
      icon: 'src/classes/modules/smview_maps/theme/img/icons/network_128.png'
    }
    evt.dataTransfer.setData('text/json', JSON.stringify(transfer))
  }

  handleFilterChange (evt: React.ChangeEvent<HTMLInputElement>) {
    const filter = evt.currentTarget.value
    let filteredItems: ShdMap[] = []
    if (filter != '') {
      for (const key in this.props.maps) {
        if (this.props.maps.hasOwnProperty(key)) {
          const object = this.props.maps[key]
          const title = object.title.toLowerCase()
          if (title.includes(filter.toLowerCase())) {
            filteredItems.push(object)
          }
        }
      }
    } else {
      filteredItems = this.props.maps
    }
    this.setState((prevState) => ({ ...prevState, filter: filter, filtered_items: filteredItems }))
  }

  render () {
    const maps = []
    let count = 0

    // sort filtered items
    const items: ShdMap[] = []
    this.state.filtered_items.forEach(object => { items.push(object) })
    items.sort((a: ShdMap, b: ShdMap) => {
      return ('' + a.title.toLowerCase()).localeCompare(b.title.toLowerCase())
    })

    items.forEach(object => {
      count++
      if (count > 10) {
        return
      }

      maps.push(<tr key={object.map_id} id={object.map_id.toString()} draggable={true} onDragStart={this.handleDragStart}>
        <td>
          <span className="icon"
            id={object.map_id.toString()}
            draggable={true}
            onDragStart={this.handleDragStart}
            title={this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__DRAG_HINT}>+</span>
        </td>
        <td>{object.title}</td>
      </tr>)

      if (count == 10) {
        maps.push(<tr key="more_objects">
          <td colSpan={2}>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__MORE_THAN_25_OBJECTS}</td>
        </tr>)
      }
    })

    if (Object.keys(this.props.maps).length == 0) {
      maps.push(<tr key="no_maps">
        <td colSpan={2}>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__NO_MAPS}</td>
      </tr>)
    }

    return <div className="tools-menu map-menu">
      <div>
        <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FILTER}:</span>
        <input type="text" value={this.state.filter} onChange={this.handleFilterChange} className="input input-small" />
      </div>
      <div>
        <table className="mainTable">
          <thead>
            <tr>
              <th>&nbsp;</th>
              <th>{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__DESCRIPTION}</th>
            </tr>
          </thead>
          <tbody>
            {maps}
          </tbody>
        </table>
      </div>
    </div>
  }
}
