import * as React from 'react'
import { NodeSetupProps, TransferObject } from '../MapsCompose'

interface ShapeMenuProps {
    node_setup: NodeSetupProps,
    onNodeSetupChange: Function,
}

export class ShapeMenu extends React.Component<ShapeMenuProps, {}> {
    static readonly SHAPES: {[key: number]: string} = {
      0: 'shape-rectangle',
      1: 'shape-round_rectangle',
      2: 'shape-ellipse',
      3: 'shape-rhomboid',
      4: 'shape-hexagon',
      5: 'shape-triangle',
      7: 'shape-stop_sign',
      8: 'shape-rhomb',
      9: 'shape-trapez_top',
      10: 'shape-trapez_bottom'
    };

    constructor (props: ShapeMenuProps) {
      super(props)
      this.handleDragStart = this.handleDragStart.bind(this)
      this.handleShapeChange = this.handleShapeChange.bind(this)
    }

    handleDragStart (evt: React.DragEvent<HTMLImageElement>) {
      const id = parseInt((evt.target as HTMLDivElement).id)
      const shape = ShapeMenu.SHAPES[id]
      const transfer: TransferObject = {
        transfer_type: 'shape',
        object_id: id,
        icon: shape
      }
      evt.dataTransfer.setData('text/json', JSON.stringify(transfer))
    }

    handleShapeChange (evt: React.MouseEvent<HTMLInputElement>) {
      const span = evt.target as HTMLSpanElement
      const nodeSetup = this.props.node_setup
      nodeSetup.shape_type = parseInt(span.id)
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    render () {
      const shapes = []
      for (const key in ShapeMenu.SHAPES) {
        if (ShapeMenu.SHAPES.hasOwnProperty(key)) {
          const shape = ShapeMenu.SHAPES[key]
          const isSelected = this.props.node_setup.shape_type && this.props.node_setup.shape_type.toString() == key
          shapes.push(<span className={'shape ' + shape + (isSelected ? ' selected' : '')}
            id={key}
            key={key}
            onClick={this.handleShapeChange}
            draggable={true}
            onDragStart={this.handleDragStart} />)
        }
      }

      return <div className="tools-menu shape-menu">
        <div>
          {shapes}
        </div>
      </div>
    }
}
