import * as React from 'react'
import { NodeSetupProps } from '../MapsCompose'

interface TextMenuProps {
    node_setup: NodeSetupProps,
    onNodeSetupChange: Function,
    language: {[key: string]: string},
}

export class TextMenu extends React.Component<TextMenuProps, {}> {
    static readonly FONTS: string[] = [
      'Arial',
      'Dialog',
      'Tahoma'
    ];

    static readonly FONT_SIZES: number[] = [8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72, 96];

    constructor (props: TextMenuProps) {
      super(props)
      this.handleFontFamilyChange = this.handleFontFamilyChange.bind(this)
      this.handleFontSizeChange = this.handleFontSizeChange.bind(this)
      this.handleIsBoldChange = this.handleIsBoldChange.bind(this)
      this.handleIsItalicChange = this.handleIsItalicChange.bind(this)
      this.handleTextColorChange = this.handleTextColorChange.bind(this)
      this.handleBackgroundColorChange = this.handleBackgroundColorChange.bind(this)
    }

    handleFontFamilyChange (evt: React.ChangeEvent<HTMLSelectElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.family = evt.currentTarget.value
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    handleFontSizeChange (evt: React.ChangeEvent<HTMLSelectElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.size = parseInt(evt.currentTarget.value)
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    handleIsBoldChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.is_bold = evt.currentTarget.checked
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    handleIsItalicChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.is_italic = evt.currentTarget.checked
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    handleTextColorChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.color = evt.currentTarget.value
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    handleBackgroundColorChange (evt: React.ChangeEvent<HTMLInputElement>) {
      const nodeSetup = this.props.node_setup
      nodeSetup.background_color = evt.currentTarget.value
      this.props.onNodeSetupChange(nodeSetup, true)
    }

    render () {
      const fonts = []
      for (const key in TextMenu.FONTS) {
        if (TextMenu.FONTS.hasOwnProperty(key)) {
          const font = TextMenu.FONTS[key]
          fonts.push(<option key={font} value={font}>{font}</option>)
        }
      }

      const fontSizes = []
      for (const key in TextMenu.FONT_SIZES) {
        if (TextMenu.FONT_SIZES.hasOwnProperty(key)) {
          const size = TextMenu.FONT_SIZES[key]
          fontSizes.push(<option key={size} value={size}>{size}</option>)
        }
      }
      return <div className="tools-menu text-menu">
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FONT_STYLE}:</span>
          <select name="font_family" value={this.props.node_setup.family} onChange={this.handleFontFamilyChange} className="input input-mini">
            {fonts}
          </select>
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FONT_SIZE}:</span>
          <select name="font_size" value={this.props.node_setup.size} onChange={this.handleFontSizeChange} className="input input-mini">
            {fontSizes}
          </select>
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_BOLD}:</span>
          <input name="is_bold"
            type="checkbox"
            checked={this.props.node_setup.is_bold}
            onChange={this.handleIsBoldChange} />
        </div>
        <div>
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_ITALIC}:</span>
          <input name="is_italic"
            type="checkbox"
            checked={this.props.node_setup.is_italic}
            onChange={this.handleIsItalicChange} />
        </div>
        <div className="color-picker">
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__TEXT_COLOR}:</span>
          <input name="text_color"
            type="color"
            onChange={this.handleTextColorChange}
            value={this.props.node_setup.color} />
        </div>
        <div className="color-picker">
          <span className="title">{this.props.language.LC__CMDB__CATG__SMVIEW_MAPS__BACKGROUND_COLOR}:</span>
          <input name="background_color"
            type="color"
            onChange={this.handleBackgroundColorChange}
            value={this.props.node_setup.background_color} />
        </div>
      </div>
    }
}
