'use strict'

import 'yfiles/view-layout-bridge.js'
import { INode, NodeStyleBase } from 'yfiles/view-component'

export abstract class AbstractNodeStyle extends NodeStyleBase {
  protected getRect (node: INode) {
    const rect = window.document.createElementNS('http://www.w3.org/2000/svg', 'rect')
    const nodeSize = node.layout.toSize()

    const w = nodeSize.width
    const h = nodeSize.height
    const x = 0 // offset from node
    const y = 0 // offset from node

    rect.setAttribute('x', x.toString())
    rect.setAttribute('y', y.toString())
    rect.setAttribute('width', w.toString())
    rect.setAttribute('height', h.toString())

    // determine the color to use for the rendering
    const color = this.getNodeColor(node)
    rect.setAttribute('fill', color)
    return rect
  }

  protected getNodeColor (node: INode) {
    return this._getColor(node, 'fillcolor', this.getDefaultFillColor())
  }

  protected getBorderColor (node: INode) {
    return this._getColor(node, 'linecolor', this.getDefaultBorderColor())
  }

  protected _getColor (node: INode, property: string, defaultColor: string) {
    const data = node.tag
    if (!data || !data[property]) {
      return defaultColor
    }
    const value = data[property] as string
    if (value.length != 7) {
      // no hex value
      return defaultColor
    }
    return value
  }

    abstract getDefaultFillColor(): string;
    abstract getDefaultBorderColor(): string;
}
