'use strict'

import 'yfiles/view-layout-bridge.js'
import { INode, IRenderContext, NodeStyleBase, Rect, SimpleNode } from 'yfiles/view-component'
import { ImageNodeStyle } from 'yfiles/styles-other'
import { SvgVisual, SvgVisualGroup } from 'yfiles'

export class MyImageNodeStyle extends NodeStyleBase {
    private readonly baseUrl: string;
    private imageNode: SimpleNode;

    constructor (baseUrl: string) {
      super()
      this.baseUrl = baseUrl

      this.imageNode = new SimpleNode()
      this.imageNode.layout = new Rect(0, 0, 64, 64)
    }

    getImageStyle (node: INode) {
      const data = node.tag
      let imageStyle
      if (data.link && data.link != '') {
        // load map preview
        // return this.baseUrl + 'upload/images/sm-view/maps/' + data.link + '.jpg';
        imageStyle = new ImageNodeStyle()
        imageStyle.image = this.baseUrl + 'src/classes/modules/smview_maps/theme/img/icons/network_128.png'
      } else {
        if (data.image_file) {
          // base64 encoded string, prepared on server
          const mimeType = data.image_mime_type ? data.image_mime_type : 'png'
          const dataUri = 'data:image/' + mimeType + ';base64,' + data.image_file
          imageStyle = new ImageNodeStyle(dataUri)
        } else {
          const url = data.imagepath ? data.imagepath.replace(this.baseUrl, '') : ''
          imageStyle = new ImageNodeStyle()
          imageStyle.image = this.baseUrl + url
        }
      }
      return imageStyle
    }

    static getImageNode (node: INode) {
      const data = node.tag
      const imageNode = new SimpleNode()

      imageNode.layout = new Rect(0, 0, 64, 64)
      if (data) {
        const pos = data.position.split(',')
        const x = pos[0]
        const y = pos[1]
        imageNode.layout = new Rect(x, y, data.width, data.height)
      }
      return imageNode
    }

    createVisual (context: IRenderContext, node: INode) {
      const layout = node.layout.toRect()

      // create the decoration
      const imageStyle = this.getImageStyle(node)
      this.imageNode = MyImageNodeStyle.getImageNode(node)
      this.imageNode.layout = this.getImageNodeLayout(layout)
      const decorationRenderer = imageStyle.renderer.getVisualCreator(this.imageNode, imageStyle)
      const decorationVisual = decorationRenderer.createVisual(context) as SvgVisual

      const group = new SvgVisualGroup()
      group.add(decorationVisual)
      return group
    }

    updateVisual (context: IRenderContext, oldVisual: SvgVisual, node: INode) {
      const imageStyle = this.getImageStyle(node)
      return imageStyle.renderer.getVisualCreator(node, imageStyle).updateVisual(context, oldVisual)
    }

    getImageNodeLayout (nodeLayout: Rect) {
      const size = this.imageNode.layout.toSize()
      return new Rect(nodeLayout.x, nodeLayout.y, size.width, size.height)
    }
}
