'use strict'

import 'yfiles/view-layout-bridge.js'
import { INode, IRenderContext, SvgVisual } from 'yfiles/view-component'
import { AbstractNodeStyle } from './AbstractNodeStyle'

export class TextNodeStyle extends AbstractNodeStyle {
  createVisual (context: IRenderContext, node: INode) {
    // This implementation creates a 'g' element and uses it as a container for the rendering of the node.
    const g = window.document.createElementNS('http://www.w3.org/2000/svg', 'g')
    // Render the node
    this.render(context, node, g)
    // set the location
    SvgVisual.setTranslate(g, node.layout.x, node.layout.y)
    return new SvgVisual(g)
  }

  render (context: IRenderContext, node: INode, container: Element) {
    // Create Defs section in container
    const defs = window.document.createElementNS('http://www.w3.org/2000/svg', 'defs')
    container.appendChild(defs)

    const shape = this.getRect(node)

    // set border color
    // shape.setAttribute('stroke-width', '0.8')
    // shape.setAttribute('stroke', this.getBorderColor(node))

    if (shape) {
      container.appendChild(shape)
    }
  }

  getDefaultFillColor () {
    return '#FFFFFF'
  }

  getDefaultBorderColor () {
    return '#000000'
  }
}
