const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const HtmlWebpackPlugin = require('html-webpack-plugin');

module.exports = {
    entry: [
        './js/src/index.tsx',
        './sass/style.scss'
    ],
    output: {
        filename: 'maps.[contenthash].js',
        path: resolve(__dirname, './dist/'),
        publicPath: 'src/classes/modules/smview_maps/assets/dist/',
    },
    resolve: {
        cacheWithContext: false,
        extensions: ['.ts', '.tsx', '.js'],
        symlinks: false,
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                include: resolve(__dirname, 'js/src'),
                use: 'ts-loader',
                exclude: /node_modules/,
            },
            // sass to css file
            {
                test: /\.(scss)$/,
                include: resolve(__dirname, 'sass'),
                use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
                sideEffects: true
            },
            {
                test: /\.(png|svg|jpg|gif)$/,
                include: resolve(__dirname, 'icons'),
                loader: 'file-loader',
                options: {
                    outputPath: 'images',
                    publicPath: '../images'
                }
            }
        ]
    },
    optimization: {
        minimize: false,
        // separate 3rd-party libs because they won't change so often
        moduleIds: 'hashed',
        chunkIds: 'named',
        splitChunks: {
            chunks: 'async',
            maxAsyncRequests: 6,
            maxInitialRequests: 4,
            cacheGroups: {
                yfiles: {
                    test: /[\\/]yfiles[\\/]/,
                    name: 'yfiles',
                    chunks: 'all',
                    priority: 10
                },
                vendors: {
                    test: /[\\/]node_modules[\\/]/,
                    name: 'vendors',
                    chunks: 'all'
                }
            }
        }
    },
    plugins: [
        new CleanWebpackPlugin(),
        new HtmlWebpackPlugin({
            path: resolve(__dirname, './dist/'),
            inject: false,
            templateContent: ({htmlWebpackPlugin}) => `${htmlWebpackPlugin.tags.headTags} ${htmlWebpackPlugin.tags.bodyTags}`
        }),
        new MiniCssExtractPlugin({
            filename: 'css/maps-style.css'
        }),
        new CopyWebpackPlugin([
            {from: 'css', to: 'css'},
            {from: 'js/lib/license.js', to: 'license23.js'}
        ])
    ]
};
