const { CleanWebpackPlugin } = require('clean-webpack-plugin');
const { resolve } = require('path');
const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');

module.exports = function(env, options) {
  const config = {
    entry: [
      './js/src/index.tsx',
      './sass/style.scss'
    ],
    output: {
      filename: 'maps.js',
      path: resolve(__dirname, './dist/')
    },
    resolve: {
      extensions: ['.ts', '.tsx', '.js', '.json'],
    },
    module: {
      rules: [
        {
          test: /\.tsx?$/,
          loader: 'awesome-typescript-loader'
        },
        // sass to css file
        {
          test: /\.(scss)$/,
          use: [MiniCssExtractPlugin.loader, 'css-loader', 'sass-loader'],
          sideEffects: true
        },
        {
          test: /\.(png|svg|jpg|gif)$/,
          loader: 'file-loader',
          options: {
            outputPath: 'images',
            publicPath: '../images'
          }
        }
      ]
    },
    optimization: {
      minimize: false,
      // separate 3rd-party libs because they won't change so often
      moduleIds: 'hashed',
      chunkIds: 'named',
      splitChunks: {
        cacheGroups: {
          yfiles: {
            test: /[\\/]yfiles[\\/]/,
            name: 'yfiles',
            chunks: 'all',
            priority: 10
          },
          vendors: {
            test: /[\\/]node_modules[\\/]/,
            name: 'vendors',
            chunks: 'all'
          }
        }
      }
    },
    plugins: [
      new CleanWebpackPlugin(),
      new MiniCssExtractPlugin({
        filename: 'css/maps-style.css'
      }),
      new CopyWebpackPlugin([
        {from: 'css', to: 'css'},
        {from: 'js/lib/license.js', to: 'license.js'}
      ])
    ]
  };
  return config
};
