<?php

use idoit\Component\Property\Type\DialogProperty;
use idoit\Component\Property\Type\TextProperty;

/**
 * i-doit
 *
 * DAO: global category for SM-VIEW Maps Assigned Maps.
 *
 * @package    i-doit
 * @subpackage smview_maps
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_dao_category_g_smview_maps_assigned_maps extends isys_cmdb_dao_category_global
{
    /**
     * Category's name. Will be used for the identifier, constant, main table, and many more.
     *
     * @var string
     */
    protected $m_category = 'smview_maps_assigned_maps';

    /**
     * Is category multi-valued or single-valued?
     *
     * @var boolean
     */
    protected $m_multivalued = true;

    /**
     * Method for returning the properties.
     *
     * @return array
     */
    protected function properties()
    {
        return [
            'description' => (new TextProperty(
                'C__CATG__SMVIEW_MAPS_DESCRIPTION',
                'LC__UNIVERSAL__DESCRIPTION',
                'isys_catg_smview_maps_assigned_maps_list__description',
                'isys_catg_smview_maps_assigned_maps_list'
            ))->mergePropertyUiParams(
                [
                    'p_strClass'  => 'input',
                ]
            ),
            'map_id' => (new DialogProperty(
                'C__CATG__SMVIEW_MAPS_MAP_ID',
                'LC__CMDB__CATG__SMVIEW_MAPS__MAP_NAME',
                'isys_catg_smview_maps_assigned_maps_list__map_id',
                'isys_catg_smview_maps_assigned_maps_list',
                'shd_map'
            ))
        ];
    }

    /**
     * Retrieves the number of saved category-entries to the given object.
     *
     * @param  int|null $objectId
     * @return int
     * @throws isys_exception_database
     */
    public function get_count($objectId = null)
    {
        if ($objectId === null || $objectId <= 0) {
            $objectId = $this->m_object_id;
        }

        if ($objectId > 0) {
            $res = $this->get_data(null, $objectId);
            return $res->count();
        }
        return false;
    }

    /**
     * Fetches category data from database.
     *
     * @param  int|null $p_category_data_id
     * @param  int|null $p_obj_id
     * @param  string $p_condition
     * @param  array|null $p_filter
     * @param  int|null $p_status
     * @return isys_component_dao_result
     * @throws isys_exception_database
     */
    public function get_data($p_category_data_id = null, $p_obj_id = null, $p_condition = '', $p_filter = null, $p_status = null)
    {
        // get manually assigned maps
        $l_sql = 'SELECT
                    l.isys_catg_smview_maps_assigned_maps_list__id AS list_id,
                    l.isys_catg_smview_maps_assigned_maps_list__isys_obj__id AS obj_id,
                    "manual" AS type,
                    l.isys_catg_smview_maps_assigned_maps_list__description as description,
                    m.shd_map__id as map_id,
                    m.shd_map__title as map_title
            FROM isys_catg_smview_maps_assigned_maps_list l
            JOIN shd_map m ON l.isys_catg_smview_maps_assigned_maps_list__map_id = m.shd_map__id
			WHERE l.isys_catg_smview_maps_assigned_maps_list__isys_obj__id = ' . $this->convert_sql_id($p_obj_id);

        $l_cRecStatus = empty($p_cRecStatus) ? 2 : $p_cRecStatus;
        if (!empty($l_cRecStatus)) {
            $l_sql .= ' AND l.isys_catg_smview_maps_assigned_maps_list__status = ' . $l_cRecStatus;
        }
        if (!empty($p_category_data_id)) {
            $l_sql .= ' AND l.isys_catg_smview_maps_assigned_maps_list__map_id = ' . $this->convert_sql_id($p_category_data_id);
        }

        // add maps, which uses this object within the map
        $l_sql .= ' UNION ';
        $l_sql .= 'SELECT
                    -1 AS list_id,
                    o.shd_map_mapobject__isys_obj__id AS obj_id,
                    "implicit" AS type,
                    "" as description,
                    m.shd_map__id as map_id,
                    m.shd_map__title as map_title
            FROM shd_map m
            JOIN shd_map_mapobject o ON m.shd_map__id = o.shd_map_mapobject__shd_map__id
			WHERE o.shd_map_mapobject__isys_obj__id = ' . $this->convert_sql_id($p_obj_id);

        if (!empty($p_category_data_id)) {
            $l_sql .= ' AND m.shd_map__id = ' . $this->convert_sql_id($p_category_data_id);
        }
        return $this->retrieve($l_sql . ';');
    }

    /**
     * Change the status of an object (such as server, switch, monitor, ...).
     *
     * @param   integer $p_object_id
     * @param   integer $p_direction
     * @param   string  $p_table
     * @param   array   $p_checkMethod
     * @param   boolean $p_purge
     * @return  boolean
     * @throws  Exception
     * @throws  isys_exception_cmdb
     * @throws  isys_exception_dao
     * @throws  isys_exception_database
     * @throws  isys_exception_general
     */
    public function rank_record($p_object_id, $p_direction, $p_table, $p_checkMethod = null, $p_purge = false)
    {
        // $p_object_id is set to map_id in isys_cmdb_dao_list_catg_smview_maps_assigned_maps
        // try to get the list item by its object- and map-ID to call rank_record with a valid list ID
        $l_sql = 'SELECT l.isys_catg_smview_maps_assigned_maps_list__id AS list_id
            FROM isys_catg_smview_maps_assigned_maps_list l
			WHERE l.isys_catg_smview_maps_assigned_maps_list__isys_obj__id = ' . $this->convert_sql_id($this->m_object_id) . '
			AND l.isys_catg_smview_maps_assigned_maps_list__map_id = ' . $this->convert_sql_id($p_object_id);
        $row = $this->retrieve($l_sql . ';')->get_row();
        if ($row) {
            return parent::rank_record($row['list_id'], $p_direction, $p_table, $p_checkMethod, $p_purge);
        }
        return false;
    }
}
