<?php

/**
 * i-doit
 *
 * UI: global category for SM-VIEW Maps Assigned Maps.
 *
 * @package    i-doit
 * @subpackage smview_maps
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2021 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class isys_cmdb_ui_category_g_smview_maps_assigned_maps extends isys_cmdb_ui_category_global
{

    /**
     * Overwriting this method to set the correct template.
     *
     * @param isys_cmdb_dao_category $p_cat
     * @throws Exception
     */
    public function init(isys_cmdb_dao_category $p_cat)
    {
        parent::init($p_cat);

        switch ($_POST[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
            case defined_or_default('C__NAVMODE__EDIT'):
                $this->set_template(isys_module_smview_maps::get_tpl_dir() . 'module/cmdb/catg__smview_maps_assigned_maps.tpl');
                break;
            default:
                $this->set_template(isys_module_smview_maps::get_tpl_dir() . 'map.tpl');
        }
    }

    /**
     * Processes the UI for the category
     *
     * @param  isys_cmdb_dao_category $p_cat The corresponding category DAO
     * @throws isys_exception_dao_cmdb
     * @throws isys_exception_database
     * @throws Exception
     */
    public function process(isys_cmdb_dao_category $p_cat)
    {
        parent::process($p_cat);
        $this->deactivate_commentary();
        $this->disable_ajax_save();

        switch ($_POST[C__GET__NAVMODE]) {
            case defined_or_default('C__NAVMODE__NEW'):
            case defined_or_default('C__NAVMODE__EDIT'):
                // render map assignment
                $data = $p_cat->get_general_data();
                $dao = new isys_cmdb_dao_category_g_smview_maps_assigned_maps($this->m_database_component);
                $l_sql = 'SELECT * FROM isys_catg_smview_maps_assigned_maps_list l
                    WHERE isys_catg_smview_maps_assigned_maps_list__isys_obj__id = ' . $this->m_object_id;
                $res2 = $dao->retrieve($l_sql);
                $assigned_maps = [];
                while ($row = $res2->get_row()) {
                    $map_id = $row['isys_catg_smview_maps_assigned_maps_list__map_id'];
                    $assigned_maps[$map_id] = $map_id;
                }

                // add available maps
                $maps = [];
                $l_sql = 'SELECT * FROM shd_map';
                $res = $dao->retrieve($l_sql);
                while ($row = $res->get_row()) {
                    if (! isset($assigned_maps[$row['shd_map__id']])) {
                        $maps[$row['shd_map__id']] = $row['shd_map__title'];
                    }
                }

                $rules = [];
                if ($data) {
                    if (! isset($maps[$data['map_id']])) {
                        // add selected map
                        $maps[$data['map_id']] = $data['map_title'];
                    }
                    $rules['C__CATG__SMVIEW_MAPS_MAP_ID']['p_strSelectedID'] = $data['map_id'];
                    $rules['C__CATG__SMVIEW_MAPS_MAP_ID']['p_bDisabled'] = true;
                    $rules['C__CATG__SMVIEW_MAPS_DESCRIPTION']['p_strValue'] = $data['description'];
                }

                $rules['C__CATG__SMVIEW_MAPS_MAP_ID']['p_arData'] = serialize($maps);
                $this->get_template_component()->smarty_tom_add_rules('tom.content.bottom.content', $rules);
                break;
            case defined_or_default('C__NAVMODE__SAVE'):
                // set category_data_id to current map_id to load this map after save
                $l_sql = 'SELECT
                    l.isys_catg_smview_maps_assigned_maps_list__map_id AS map_id
                    FROM isys_catg_smview_maps_assigned_maps_list l
                    WHERE l.isys_catg_smview_maps_assigned_maps_list__id = ' . $p_cat->convert_sql_id($this->m_category_data_id);
                $row = $p_cat->retrieve($l_sql)->get_row();
                $this->set_category_data_id($row['map_id']);
                // fall through
            default:
                // render selected map
                $js_config = self::get_map_configuration($this->m_category_data_id);
                $this->get_template_component()
                    ->assign('assets_dir', isys_module_smview_maps::get_assets_www_dir())
                    ->assign('js_config', json_encode($js_config));
        }
    }

    /**
     * Get analyzer configuration.
     *
     * @param  int|null $map_id
     * @return array
     * @throws isys_exception_database
     */
    public static function get_map_configuration($map_id = null)
    {
        $lang = isys_application::instance()->container->get('language');
        $l_map = isys_module_smview_maps::getMap($map_id);

        $dao = new isys_cmdb_dao(isys_application::instance()->database);
        $types = $dao->get_object_types();
        $object_types = [];
        while ($row = $types->get_row()) {
            $object_types[] = [
                'id'    => $row['isys_obj_type__id'],
                'title' => $lang->get($row['isys_obj_type__title']),
                'icon'  => $row['isys_obj_type__icon'],
            ];
        }

        return [
            'base_url'     => isys_helper_link::get_base(),
            'base_path'    => isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_MAPS'),
                    'request'         => isys_module_smview_maps::REQUEST_MAP
                ]
            ),
            'data'         => json_encode($l_map),
            'object_types' => $object_types,
            'maps'         => smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps(),
            'api_url'      => isys_helper_link::get_base() . 'src/jsonrpc.php',
            'session'      => $_COOKIE['PHPSESSID'],
            'api_key'      => self::_getApiKey(),
            'language'     => self::_get_js_translations(),
        ];
    }

    /**
     * Get API key.
     *
     * @return string
     */
    private static function _getApiKey()
    {
        $sysDb = isys_application::instance()->database_system;
        $session = isys_application::instance()->session;

        $mandatorId = (int) $session->get_mandator_id();
        $data = $sysDb->fetch_row_assoc($sysDb->query(
            "SELECT isys_mandator__apikey apiKey FROM isys_mandator WHERE isys_mandator__id = $mandatorId;"
        ));
        return $data['apiKey'];
    }

    /**
     * Get translated strings to use with javascript.
     *
     * @return array
     * @throws Exception
     */
    private static function _get_js_translations()
    {
        // language templates
        $language_templates = [
            'LC__CMDB__CATG__SMVIEW_MAPS__AUTO_RELOAD_PAGE',
            'LC__CMDB__CATG__SMVIEW_MAPS__OVERVIEW',
            'LC__CMDB__CATG__SMVIEW_MAPS__STRUCTURE',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MAP',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_CURSOR',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_NODE',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_SHAPE',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_TEXT',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_IMAGE',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_MAP',
            'LC__CMDB__CATG__SMVIEW_MAPS__EDIT_MODE_EDGE',
            'LC__CMDB__CATG__SMVIEW_MAPS__STROKE',
            'LC__CMDB__CATG__SMVIEW_MAPS__STYLE',
            'LC__CMDB__CATG__SMVIEW_MAPS__SOURCE_ARROW',
            'LC__CMDB__CATG__SMVIEW_MAPS__TARGET_ARROW',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_NONE',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_DEFAULT',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_TRIANGLE',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_DIAMOND',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_SHORT',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_SIMPLE',
            'LC__CMDB__CATG__SMVIEW_MAPS__ARROW_CIRCLE',
            'LC__CMDB__CATG__SMVIEW_MAPS__FONT_STYLE',
            'LC__CMDB__CATG__SMVIEW_MAPS__FONT_SIZE',
            'LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_BOLD',
            'LC__CMDB__CATG__SMVIEW_MAPS__FONT_IS_ITALIC',
            'LC__CMDB__CATG__SMVIEW_MAPS__LINE_COLOR',
            'LC__CMDB__CATG__SMVIEW_MAPS__TEXT_COLOR',
            'LC__CMDB__CATG__SMVIEW_MAPS__BACKGROUND_COLOR',
            'LC__CMDB__CATG__SMVIEW_MAPS__IMAGE_SIZE',
            'LC__CMDB__CATG__SMVIEW_MAPS__OBJECT_TYPE',
            'LC__CMDB__CATG__SMVIEW_MAPS__FILTER',
            'LC__CMDB__CATG__SMVIEW_MAPS__DESCRIPTION',
            'LC__CMDB__CATG__SMVIEW_MAPS__PATH',
            'LC__CMDB__CATG__SMVIEW_MAPS__STATUS',
            'LC__CMDB__CATG__SMVIEW_MAPS__CREATED',
            'LC__CMDB__CATG__SMVIEW_MAPS__CHANGED',
            'LC__CMDB__CATG__SMVIEW_MAPS__DRAG_HINT',
            'LC__CMDB__CATG__SMVIEW_MAPS__OBJECT_TYPE_CONTAINS_NO_OBJECTS',
            'LC__CMDB__CATG__SMVIEW_MAPS__NO_IMAGES',
            'LC__CMDB__CATG__SMVIEW_MAPS__NO_MAPS',
            'LC__CMDB__CATG__SMVIEW_MAPS__MORE_THAN_25_OBJECTS',
            'LC__CMDB__CATG__SMVIEW_MAPS__PREVIEW',
            'LC__CMDB__CATG__SMVIEW_MAPS__PLACEHOLDER_EMPTY_SHAPE',
        ];

        // translate
        $language = [];
        $lang = isys_application::instance()->container->get('language');
        foreach ($language_templates as $l) {
            $language[$l] = $lang->get($l);
        }
        return $language;
    }

    /**
     * Fetches category's title from database.
     *
     * @param  isys_cmdb_dao_category &$p_cat Category's DAO
     * @return string  Category's title.
     * @throws Exception
     */
    public function gui_get_title(isys_cmdb_dao_category &$p_cat)
    {
        $lang = isys_application::instance()->container->get('language');
        return $lang->get('LC__CMDB__CATG__SMVIEW_MAPS__ASSIGNED_MAPS_TITLE');
    }
}
