<?php

/**
 * Test data generator controller.
 *
 * @package    i-doit
 * @subpackage smview_maps
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2020 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
class isys_handler_smview_maps_test_data_generator extends isys_handler
{
    /**
     * Initialization method.
     *
     * You can call this controller via CLI from i-doit root:
     * sudo -u www-data ./controller -v -d -u admin -p admin -i 1 -m smview_maps_test_data_generator
     *
     * @return void
     */
    public function init()
    {
        verbose("start\n");

        verbose("add map\n");
        $map = [
            "user_id" => 1,
            "title" => 'MyMap',
        ];

        $mapApi = new isys_api_model_shd_map();
        $m = $mapApi->create($map);
        $mapId = $m['response'];

        verbose("add map objects\n");
        $objects = [
            [
                "shd_map__id" => $mapId,
                "isys_obj__id" => '',
                "position" => '10,140',
                "width" => '64',
                "height" => '64',
                "title" => 'Host1',
                "labelposition" => '109',
                "imagepath" => '',
                "layerindex" => '0',
                "textvisible" => '1',
                "link" => '',
                "realizertype" => '1',
                "shapetype" => '0',
                "fillcolor" => '-4144960',
                "linecolor" => '-16777216',
                "labellinecolor" => '-16777216',
                "textcolor" => '-16777216',
                "font" => 'Dialog;0;22',
                "textalignment" => '0',
            ],
            [
                "shd_map__id" => $mapId,
                "isys_obj__id" => '',
                "position" => '40,20',
                "width" => '124',
                "height" => '64',
                "title" => 'Host2',
                "labelposition" => '109',
                "imagepath" => '',
                "layerindex" => '0',
                "textvisible" => '1',
                "link" => '',
                "realizertype" => '1',
                "shapetype" => '2',
                "fillcolor" => '-4144960',
                "linecolor" => '-16777216',
                "labellinecolor" => '-16777216',
                "textcolor" => '-16777216',
                "font" => 'Dialog;0;14',
                "textalignment" => '0',
            ]
        ];

        $mapObjectApi = new isys_api_model_shd_map_object();
        $mapObjects = [];
        foreach ($objects as $object) {
            $o = $mapObjectApi->create($object);
            $mapObjects[$object['title']] = $o['response'];
        }

        verbose("add map connections\n");
        $connection = [
            "shd_map__id" => $mapId,
			"isys_obj__id" => '',
			"title" => 'Edge1',
			"source_shd_map_mapobject__id" => $mapObjects['Host1'],
			"target_shd_map_mapobject__id" => $mapObjects['Host2'],
			"sourcearrowtype" => '1',
			"targetarrowtype" => '2',
			"sourcepoint" => '0,0',
			"targetpoint" => '0,0',
			"bends" => '',
			"linestyle" => '0',
			"linewidth" => '3',
			"linecolor" => '-16777216',
			"textcolor" => '-16777216',
			"font" => 'Dialog;0;14',
			"layerindex" => '0',
			"realizertype" => '0',
			"connectiontype" => '',
			"textvisible" => '0',
        ];
        
        $mapConnectionApi = new isys_api_model_shd_map_connection();
        $mapConnectionApi->create($connection);

        verbose("finished\n");
    }
}
