<?php

/**
 * i-doit
 *
 * SM-VIEW Maps module.
 *
 * @package    idoit
 * @subpackage smview_maps
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2018 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license MIT
 */
final class isys_module_smview_maps extends isys_module
{

    const DISPLAY_IN_MAIN_MENU = true;
    const DISPLAY_IN_SYSTEM_MENU = false;
    const REQUEST_MAP = 'map';

    /**
     * @var bool
     */
    protected static $m_licenced = true;

    /**
     * Variable which holds the template component.
     *
     * @var isys_component_template
     */
    protected $m_tpl;
    
    /**
     * Variable which holds the template component.
     *
     * @var isys_component_tree
     */
    protected $m_tree;

    /**
     * Static method for retrieving the path, to the modules templates.
     *
     * @static
     * @return string
     * @author Leonard Fischer <lfischer@i-doit.com>
     */
    public static function get_tpl_dir() 
    {
        return __DIR__ . '/templates/';
    }

    /**
     * Represents the name, of the corresponding auth class - "false" means, there is none.
     *
     * @var string
     */
    const AUTH_CLASS_NAME = 'isys_auth_module_smview_maps';

    /**
     * Method for retrieving the path to the module directory (needed for includes).
     *
     * @static
     * @return string
     */
    public static function get_module_dir() 
    {
        return __DIR__ . DS;
    }
    
    /**
     * Method for retrieving the www-path to the assets directory (needed for the frontend).
     *
     * @static
     * @return string
     */
    public static function get_assets_www_dir() 
    {
        return isys_application::instance()->www_path . 'src/classes/modules/smview_maps/assets/';
    }

    /**
     * Initializes the module.
     *
     * @param  isys_module_request & $p_req
     * @return isys_module_smview_maps
     */
    public function init(isys_module_request &$p_req) 
    {
        $this->m_tpl = $p_req->get_template();
        $this->m_tree = $p_req->get_menutree();

        return $this;
    }

    /**
     * Start method.
     *
     * @desc   Starts module process
     * @throws Exception
     */
    public function start() 
    {
        global $g_dirs;

        $lang = isys_application::instance()->container->get('language');
        $l_template = $this->m_tpl;
        $l_tree = $this->m_tree;
        $l_tree->add_node(0, -1, $lang->get("LC__CMDB__CATG__SMVIEW_MAPS__TREE_TITLE"),
            isys_helper_link::create_url(
                [
                    C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_MAPS')
                ]
            ));
        
        $requiredClasses = array(
            'smview_cmdb_dao_map'             => 'SM-VIEW',
            'isys_cache_livestatus_memcached' => 'Livestatus',
            'isys_cmdb_dao_profile'           => 'BSM',
        );
        
        foreach ($requiredClasses as $className => $moduleName) {
            if (! class_exists($className)) {
                $l_template->assign(
                    'error', array(
                        'message' => sprintf($lang->get('LC__CMDB__CATG__SMVIEW_MAPS__ADD_ON_NOT_INSTALLED'), $moduleName),
                        'hint'    => $lang->get('LC__CMDB__CATG__SMVIEW_MAPS__SMVIEW_INSTALL_TIP'),
                    )
                );
                $l_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/overview.tpl');
                return;
            }
        }
        
        $l_maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps();
        $childMaps = [];
        $titleMaps = [];
        $children = [];
        foreach ($l_maps as $map) {
            $map_id = $map['map_id'];
            $titleMaps[$map_id] = $map['title'];
            $l_map = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMap($map_id);

            foreach ($l_map['map_link'] as $map_link) {
                $childMaps[$map_id][] = $map_link;
                $children[$map_link] = $map_link;
            }
        }
        
        // add root nodes
        foreach ($l_maps as $map) {
            $map_id = $map['map_id'];
            if (isset($children[$map_id])) {
                continue;
            }
            
            $l_tree->add_node(
                $map_id, 0, $map['title'], 
                isys_helper_link::create_url(
                    [
                        C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_MAPS'),
                        C__GET__TREE_NODE => $map_id,
                        'request'         => self::REQUEST_MAP,
                        'instance_id'     => $map_id,
                    ]
                ),
                '',
                $g_dirs['images'] . 'icons/silk/world.png'
            );
        }
        
        // add child nodes
        foreach (array_keys($childMaps) as $parent_id) {
            $this->_addChildren($l_tree, $childMaps, $parent_id, $titleMaps);
        }
        
        $l_template->assign("menu_tree", $l_tree->process($_GET[C__GET__TREE_NODE]));
        
        if (isset($_GET['request'])) {
            $instance_id = isset($_GET['instance_id']) ? $_GET['instance_id'] : null;
            $this->route($_GET['request'], $instance_id);
        } else {
            $this->route('/');
        }
    }
    
    /**
     * Add child nodes.
     * 
     * @param  isys_component_tree $l_tree
     * @param  array               $childMaps
     * @param  int                 $parent_id
     * @param  array               $titleMaps
     */
    private function _addChildren($l_tree, $childMaps, $parent_id, $titleMaps) 
    {
        global $g_dirs;
        
        if (! isset($childMaps[$parent_id])) {
            return;
        }
        
        $children = $childMaps[$parent_id];
        foreach ($children as $child_id) {
            $title = $titleMaps[$child_id];
            $l_tree->add_node(
                $child_id, $parent_id, $title,
                isys_helper_link::create_url(
                    [
                        C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_MAPS'),
                        C__GET__TREE_NODE => $child_id,
                        'request'         => self::REQUEST_MAP,
                        'instance_id'     => $child_id,
                    ]
                ),
                '',
                $g_dirs['images'] . 'icons/silk/world_link.png'
            );
            
            if (isset($childMaps[$child_id])) {
                $this->_addChildren($l_tree, $childMaps, $child_id, $titleMaps);
            }
        }
    }

    /**
     * Routing.
     *
     * @param  string $p_request
     * @param  int|null $p_instance_id
     * @throws Exception
     */
    public function route($p_request, $p_instance_id = null) 
    {
        $lang = isys_application::instance()->container->get('language');
        $l_template = $this->m_tpl;
        $l_template->assign('assets_dir', $this->get_assets_www_dir());
        $l_template->assign('title', $lang->get('LC__CMDB__CATG__SMVIEW_MAPS__TREE_TITLE'));
        switch ($p_request) {
            case self::REQUEST_MAP:
                $js_config = isys_cmdb_ui_category_g_smview_maps_assigned_maps::get_map_configuration($p_instance_id);
                $icon_dir = isys_helper_link::get_base() . 'src/classes/modules/smview_maps/theme/img/icons/';
                $l_template->assign('js_config', json_encode($js_config));
                $l_template->assign('icon_dir', $icon_dir);
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/map.tpl');
                break;
            default:
                global $g_comp_session;
                $userID = $g_comp_session->get_user_id();

                $l_posts = isys_module_request::get_instance()->get_posts();
                if ($l_posts[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__SAVE')) {
                    if (! isset($l_posts['C__CATG__SMVIEW_MAPS_NAME'])) {
                        return;
                    }

                    $name = $l_posts['C__CATG__SMVIEW_MAPS_NAME'];
                    $mapId = smview_cmdb_dao_map::instance(isys_application::instance()->database)->createMap($name, $userID);
                    $l_template->assign('redirect_url', isys_helper_link::create_url(
                        [
                            C__GET__MODULE_ID => defined_or_default('C__MODULE__SMVIEW_MAPS'),
                            C__GET__TREE_NODE => $mapId,
                            'request'         => self::REQUEST_MAP,
                            'instance_id'     => $mapId,
                        ]
                    ));
                }

                $l_navbar = isys_component_template_navbar::getInstance();
                if ($_POST[C__GET__NAVMODE] == defined_or_default('C__NAVMODE__NEW')) {
                    $l_navbar->set_selected(true, defined_or_default('C__NAVBAR_BUTTON__NEW'))
                        ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__SAVE'))
                        ->set_active(true, defined_or_default('C__NAVBAR_BUTTON__CANCEL'));
                } else {
                    $l_navbar->set_active(true, defined_or_default('C__NAVBAR_BUTTON__NEW'))
                        ->set_visible(true, defined_or_default('C__NAVBAR_BUTTON__NEW'));
                    $maps = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMaps();
                    $l_template->assign('maps', ! empty($maps) ? $maps : []);
                }
                $l_template->include_template('contentbottomcontent', __DIR__ . '/templates/overview.tpl');
                break;
        }
    }

    /**
     * Get a map by its ID.
     *
     * @param  int $p_instance_id
     * @return array
     * @throws Exception
     */
    public static function getMap($p_instance_id)
    {
        $l_map = smview_cmdb_dao_map::instance(isys_application::instance()->database)->getMap($p_instance_id);

        $api = new isys_api_model_shd_map();
        try {
            $status = $api->getMapStatus($p_instance_id);
            $statusObjects = $status['objects'];
            $statusMaps = $status['maps'];
        } catch (isys_exception_api $e) {
            $statusObjects = [];
            $statusMaps = [];
        }
                
        $objects = [];
        foreach ($l_map['map_objects'] as $obj) {
            $colorFields = [
                'fillcolor',
                'fillcolor2',
                'linecolor',
                'labellinecolor',
                'textcolor',
            ];

            // load base64 images from filesystem
            $imagePath = $obj["imagepath"];
            $type = self::_get_mime_type($imagePath);
            $obj["image_file"] = null;
            $obj["image_mime_type"] = $type;
            if ($type) {
                if (strpos($imagePath,  'http') !== 0) {
                    $obj["image_file"] = self::_get_file($imagePath);
                } else {
                    $relPath = str_replace(isys_helper_link::get_base(), '', $imagePath);
                    $obj["image_file"] = self::_get_file(null, $relPath);
                }
            }

            foreach ($colorFields as $field) {
                $obj[$field] = self::_convertColor($obj[$field]);
            }

            if ($obj['fillcolor'] == '#000000' && $obj['fillcolor'] == $obj['textcolor']) {
                // avoid black background with black text color
                $obj['fillcolor'] = '#FFFFFF';
            }

            if (isset($statusObjects[$obj['mapobject_id']])) {
                $statusObject = $statusObjects[$obj['mapobject_id']];
                $obj['status'] = $statusObject['status'];
            }

            if (! empty($obj['link']) && isset($statusMaps[$obj['link']])) {
                $statusMap = $statusMaps[$obj['link']];
                $obj['status'] = $statusMap['status'];
            }
            $objects[] = $obj;
        }
        $l_map['map_objects'] = $objects;

        $connections = [];
        foreach ($l_map['map_connection'] as $conn) {
            $colorFields = [
                'linecolor',
                'textcolor',
            ];

            foreach ($colorFields as $field) {
                $conn[$field] = self::_convertColor($conn[$field]);
            }
            $connections[] = $conn;
        }
        $l_map['map_connection'] = $connections;
        return $l_map;
    }

    /**
     * Get mime type of an image.
     *
     * @param  string $imagePath
     * @return string|null
     */
    private static function _get_mime_type($imagePath)
    {
        $mimeTypeMapping = [
            'jpeg' => 'jpeg',
            'jpg'  => 'jpeg',
            'png'  => 'png',
            'bpm'  => 'bpm',
            'gif'  => 'gif',
            'tif'  => 'tiff',
            'tiff' => 'tiff',
        ];
        $parts = explode('.', $imagePath);
        if (count($parts) == 0) {
            return null;
        }
        $type = end($parts);
        return isset($mimeTypeMapping[$type]) ? $mimeTypeMapping[$type] : null;
    }

    /**
     * Get base64 encoded file.
     *
     * @param  string $filename
     * @param  string|null $subPath
     * @return string|null
     */
    private static function _get_file($filename, $subPath = null)
    {
        global $g_absdir;
        if (! $subPath) {
            $subPath = 'upload/images/sm-view' . $filename;
        }

        $path =  $g_absdir . '/' . $subPath;
        if (! is_file($path)) {
            return null;
        }
        $content = file_get_contents($path);
        return base64_encode($content);
    }
    
    /**
     * Convert java-style color to hex color.
     * 
     * @param  string $javaCrap
     * @return string
     */
    private static function _convertColor($javaCrap)
    {
        $color = dechex((int) $javaCrap);
        $hex = '#' . substr($color, -6);
        return strtoupper($hex);
    }
}
