<?php

namespace idoit\Module\Smview_Maps\Console\Command;

use Exception;
use idoit\Console\Command\AbstractCommand;
use isys_api_model_shd_map;
use isys_api_model_shd_map_connection;
use isys_api_model_shd_map_object;
use Symfony\Component\Console\Input\InputDefinition;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Command to generate some test data.
 *
 * @package    i-doit
 * @subpackage smview_maps
 * @author     Steve Groeber <steve.groeber@shd-online.de>
 * @copyright  2023 SHD System-Haus-Dresden GmbH
 * @license    http://www.i-doit.com/license
 */
class TestDataGeneratorCommand extends AbstractCommand
{

    /**
     * Get name for command
     *
     * @return string
     */
    public function getCommandName()
    {
        return 'smview_maps_test_data_generator';
    }

    /**
     * Get description for command
     *
     * @return string
     */
    public function getCommandDescription()
    {
        return 'Command to generate some SM-VIEW maps test data.';
    }

    /**
     * Retrieve Command InputDefinition
     *
     * @return InputDefinition
     */
    public function getCommandDefinition()
    {
        return new InputDefinition();
    }

    /**
     * Checks if a command can have a config file via --config
     *
     * @return bool
     */
    public function isConfigurable()
    {
        return false;
    }

    /**
     * Execute command.
     *
     * @param  InputInterface  $input
     * @param  OutputInterface $output
     * @return int
     * @throws Exception
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        try {
            $output->writeln("start\n");

            $output->writeln("add map\n");
            $map = [
                "user_id" => 1,
                "title" => 'MyMap',
            ];

            $mapApi = new isys_api_model_shd_map();
            $m = $mapApi->create($map);
            $mapId = $m['response'];

            $output->writeln("add map objects\n");
            $objects = [
                [
                    "shd_map__id" => $mapId,
                    "isys_obj__id" => '',
                    "position" => '10,140',
                    "width" => '64',
                    "height" => '64',
                    "title" => 'Host1',
                    "labelposition" => '109',
                    "imagepath" => '',
                    "layerindex" => '0',
                    "textvisible" => '1',
                    "link" => '',
                    "realizertype" => '1',
                    "shapetype" => '0',
                    "fillcolor" => '-4144960',
                    "linecolor" => '-16777216',
                    "labellinecolor" => '-16777216',
                    "textcolor" => '-16777216',
                    "font" => 'Dialog;0;22',
                    "textalignment" => '0',
                ],
                [
                    "shd_map__id" => $mapId,
                    "isys_obj__id" => '',
                    "position" => '40,20',
                    "width" => '124',
                    "height" => '64',
                    "title" => 'Host2',
                    "labelposition" => '109',
                    "imagepath" => '',
                    "layerindex" => '0',
                    "textvisible" => '1',
                    "link" => '',
                    "realizertype" => '1',
                    "shapetype" => '2',
                    "fillcolor" => '-4144960',
                    "linecolor" => '-16777216',
                    "labellinecolor" => '-16777216',
                    "textcolor" => '-16777216',
                    "font" => 'Dialog;0;14',
                    "textalignment" => '0',
                ]
            ];

            $mapObjectApi = new isys_api_model_shd_map_object();
            $mapObjects = [];
            foreach ($objects as $object) {
                $o = $mapObjectApi->create($object);
                $mapObjects[$object['title']] = $o['response'];
            }

            $output->writeln("add map connections\n");
            $connection = [
                "shd_map__id" => $mapId,
                "isys_obj__id" => '',
                "title" => 'Edge1',
                "source_shd_map_mapobject__id" => $mapObjects['Host1'],
                "target_shd_map_mapobject__id" => $mapObjects['Host2'],
                "sourcearrowtype" => '1',
                "targetarrowtype" => '2',
                "sourcepoint" => '0,0',
                "targetpoint" => '0,0',
                "bends" => '',
                "linestyle" => '0',
                "linewidth" => '3',
                "linecolor" => '-16777216',
                "textcolor" => '-16777216',
                "font" => 'Dialog;0;14',
                "layerindex" => '0',
                "realizertype" => '0',
                "connectiontype" => '',
                "textvisible" => '0',
            ];

            $mapConnectionApi = new isys_api_model_shd_map_connection();
            $mapConnectionApi->create($connection);

            $output->writeln("finished\n");
        } catch (Exception $e) {
            $output->writeln("");
            $output->writeln($e);
            $output->writeln("");
        }
        return 0;
    }

    /**
     * Returns an array of command usages
     *
     * @return array
     */
    public function getCommandUsages(): array
    {
        return [
            'smview_maps_test_data_generator -u admin -p admin -i 1',
        ];
    }
}
